@@ -18,17 +18,22 @@
 #
 
 from papyon.gnet.constants import *
+from papyon.gnet.errors import *
 from papyon.gnet.proxy import ProxyInfos
 from papyon.gnet.message.HTTP import HTTPRequest
 from papyon.gnet.io import TCPClient
 from papyon.gnet.parser import HTTPParser
+from papyon.gnet.proxy.factory import ProxyFactory
 
 import gobject
 import base64
+import logging
 import platform
 
 __all__ = ['HTTP']
 
+logger = logging.getLogger('papyon.gnet.HTTP')
+
 
 class HTTP(gobject.GObject):
     """HTTP protocol client class."""
@@ -36,7 +41,7 @@ class HTTP(gobject.GObject):
     __gsignals__ = {
             "error" : (gobject.SIGNAL_RUN_FIRST,
                 gobject.TYPE_NONE,
-                (gobject.TYPE_ULONG,)),
+                (object,)), # IoError
 
             "response-received": (gobject.SIGNAL_RUN_FIRST,
                 gobject.TYPE_NONE,
@@ -47,7 +52,7 @@ class HTTP(gobject.GObject):
                 (object,)), # HTTPRequest
             }
 
-    def __init__(self, host, port=80, proxy=None):
+    def __init__(self, host, port=80, proxies={}):
         """Connection initialization
         
             @param host: the host to connect to.
@@ -56,18 +61,22 @@ class HTTP(gobject.GObject):
             @param port: the port number to connect to
             @type port: integer
 
-            @param proxy: proxy that we can use to connect
-            @type proxy: L{gnet.proxy.ProxyInfos}"""
+            @param proxies: proxies that we can use to connect
+            @type proxies: L{gnet.proxy.ProxyInfos}"""
         gobject.GObject.__init__(self)
-        assert(proxy is None or proxy.type == 'http') # TODO: add support for other proxies (socks4 and 5)
         self._host = host
         self._port = port
-        self.__proxy = proxy
+        self._proxies = proxies
+        self._http_proxy = None
         self._transport = None
         self._http_parser = None
         self._outgoing_queue = []
         self._waiting_response = False
 
+        if self._proxies and self._proxies.get('http', None):
+            if self._proxies['http'].type == 'http':
+                self._http_proxy = self._proxies['http']
+
         self._errored = False
         self.connect("error", self._on_self_error)
 
@@ -76,19 +85,26 @@ class HTTP(gobject.GObject):
 
     def _setup_transport(self):
         if self._transport is None:
-            if self.__proxy is not None:
-                self._transport = TCPClient(self.__proxy.host, self.__proxy.port)
+            if self._http_proxy:
+                self._transport = TCPClient(self._http_proxy.host,
+                        self._http_proxy.port)
             else:
                 self._transport = TCPClient(self._host, self._port)
-            self._http_parser = HTTPParser(self._transport)
-            self._http_parser.connect("received", self._on_response_received)
-            self._transport.connect("notify::status", self._on_status_change)
-            self._transport.connect("error", self._on_error)
-            self._transport.connect("sent", self._on_request_sent)
+                if self._proxies:
+                    self._transport = ProxyFactory(self._transport,
+                            self._proxies, 'http')
+            self._setup_parser()
         
         if self._transport.get_property("status") != IoStatus.OPEN:
             self._transport.open()
 
+    def _setup_parser(self):
+        self._http_parser = HTTPParser(self._transport)
+        self._http_parser.connect("received", self._on_response_received)
+        self._transport.connect("notify::status", self._on_status_change)
+        self._transport.connect("error", self._on_error)
+        self._transport.connect("sent", self._on_request_sent)
+
     def _on_status_change(self, transport, param):
         if transport.get_property("status") == IoStatus.OPEN:
             self._process_queue()
@@ -106,6 +122,9 @@ class HTTP(gobject.GObject):
     def _on_response_received(self, parser, response):
         if response.status >= 100 and response.status < 200:
             return
+        if not self._waiting_response:
+            logger.warning("Received response but wasn't waiting for one")
+            return
         #if response.status in (301, 302): # UNTESTED: please test
         #    location = response.headers['Location']
 
@@ -121,9 +140,15 @@ class HTTP(gobject.GObject):
         #        self._outgoing_queue[0].headers['Host'] = response.headers['Location']
         #        self._setup_transport()
         #        return
-        self._outgoing_queue.pop(0) # pop the request from the queue
-        self.emit("response-received", response)
         self._waiting_response = False
+        if len(self._outgoing_queue) > 0:
+            self._outgoing_queue.pop(0) # pop the request from the queue
+        if response.status >= 400:
+            logger.error("Received error code %i (%s) from %s:%i" %
+                (response.status, response.reason, self._host, self._port))
+            self.emit("error", HTTPError(response))
+        else:
+            self.emit("response-received", response)
         self._process_queue() # next request ?
 
     def _on_error(self, transport, error):
@@ -148,10 +173,10 @@ class HTTP(gobject.GObject):
             user_agent = GNet.NAME, GNet.VERSION, platform.system(), platform.machine()
             headers['User-Agent'] = "%s/%s (%s %s)" % user_agent
 
-        if self.__proxy is not None:
+        if self._http_proxy is not None:
             url = 'http://%s:%d%s' % (self._host, self._port, resource)
-            if self.__proxy.user:
-                auth = self.__proxy.user + ':' + self.__proxy.password
+            if self._http_proxy.user:
+                auth = self._http_proxy.user + ':' + self._http_proxy.password
                 credentials = base64.encodestring(auth).strip()
                 headers['Proxy-Authorization'] = 'Basic ' + credentials
         else:
@@ -159,3 +184,7 @@ class HTTP(gobject.GObject):
         request  = HTTPRequest(headers, data, method, url)
         self._outgoing_queue.append(request)
         self._process_queue()
+
+    def close(self):
+        if self._transport:
+            self._transport.close()
