@@ -18,7 +18,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-__all__ = ['annotations_to_dict', 'get_detailled_error']
+__all__ = ['annotations_to_dict']
 
 def annotations_to_dict(annotations):
     if annotations is None:
@@ -30,12 +30,3 @@ def annotations_to_dict(annotations):
         value = annotation.findtext("./ab:Value")
         result[key] = value
     return result
-
-def get_detailled_error(fault):
-    if fault.detail is not None:
-        errorcode = fault.detail.findtext("./ab:errorcode")
-        errorstring = fault.detail.findtext("./ab:errorstring")
-    else:
-        errorcode = fault.faultcode
-        errorstring = fault.faultstring
-    return errorcode, errorstring
